<?php
class Explain_Queries
{
  // how many queries were executed
  var $query_count = 0;
  // which queries and their count
  var $queries = array();
  // results of EXPLAIN-ed SELECTs
  var $explains = array();
  // the MDB2 instance
  var $db = false;

  // constructor that accepts MDB2 reference
  function Explain_Queries(&$db) {
    $this->db = $db;
  }

  // this method is called on every query
  function collectInfo(
    &$db,
    $scope,
    $message,
    $is_manip = null)
  {
    // increment the total number of queries
    $this->query_count++;
    // the SQL is a key in the queries array
    // the value will be the count of how
    // many times each query was executed
    @$this->queries[$message]++;
  }

  // print the debug information
  function dumpInfo()
  {
    echo "<div style=\"clear:both;\"></div>";
    echo '<h3>Queries on this page</h3>';
    echo '<pre style="white-space: pre-wrap;width:1000px;word-wrap: break-word;">';
    print_r($this->queries);
    echo '</pre>';
    
    echo '<h3>EXPLAIN-ed SELECTs</h3>';
    echo '<pre style="white-space: pre-wrap;width:1000px;word-wrap: break-word;">';
    print_r($this->explains);
    echo '</pre>';
  }

  // the method that will execute all SELECTs
  // with and without an EXPLAIN and will
  // create $this->explains array of debug
  // information
  // SHOW WARNINGS will be called after each
  // EXPLAIN for more information
  function executeAndExplain() {

    // at this point, stop debugging
    $this->db->setOption('debug', 0);
    $this->db->loadModule('Extended');

    // take the SQL for all the unique queries
    $queries = array_keys($this->queries);
    foreach ($queries AS $sql) {

      // for all SELECTs…
      $sql = trim($sql);
      if (stristr($sql,"SELECT") !== false) {// && stristr($sql,"TEMPORARY TABLE") === false){
        // note the start time
        $start_time = array_sum(
            explode(" ", microtime())
        );
        // execute query
        try {
          $this->db->query($sql);
        } catch(Exception $e) {
        }
        // note the end time
        $end_time = array_sum(
            explode(" ", microtime())
        );
        // the time the query took
        $total_time = $end_time - $start_time;

        // now execute the same query with
        // EXPLAIN EXTENDED prepended
        try {
          $explain = $this->db->getAll(
            'EXPLAIN EXTENDED ' . $sql
          );
        } catch(Exception $e) {
        }

        $this->explains[$sql] = array();
        // update the debug array with the
        // new data from
        // EXPLAIN and SHOW WARNINGS
        if (!PEAR::isError($explain)) {
          $this->explains[$sql]['explain'] = $explain;
          $this->explains[$sql]['warnings'] =
               $this->db->getAll('SHOW WARNINGS');
        }

        // update the debug array with the
        // count and time
        $this->explains[$sql]['time'] = $total_time;
      }
    }
  }
}

?>
