<?php

/**
 * Viper2 Framework
 *
 * @author fish <fish@archa.cz>
 * @package Viper2
 * @version 0.1
 *
 */
/**
 * default plugin
 * @package V2Default
 * @author fish <fish@archa.cz>
 * @version 0.1
 *
 * plugin se stara o defaultni zobrazeni obsahu - homepage, 404 atp.
 * zaroven definuje zakladni modifikatory a filtry pro smarty
 *
 */

class V2Default extends Viper2Plugin {
  
  /**
   * variable $pageTitle
   * @protected string titulek stranky
   */
  protected $pageTitle = "";
  /**
   * variable $pageDescription
   * @protected string popis stranky
   */
  protected $pageDescription = "";
  /**
   * variable $pageId
   * @protected string ID webu
   */
  protected $pageId = "page_id";
  /**
   * variable $attPath
   * @protected string cesta k priloham relativne k basePage
   */
  protected $attPath = "data/";
  
  public function setPageDescription($description) {
    $this->pageDescription=$description;
  }
  public function setPageTitle($title) {
    $this->pageTitle=$title;
  }
  public function setPageId($id) {
    $this->pageId=$id;
  }
  public function setAttPath($path) {
    $this->attPath=$path;
  }
  
  function __construct() {
    $this->pluginName = "Default";
    $this->pluginUri = "/";
    $this->setPriority(-999999999);
    
    if (Viper2::$useSmarty) {
      Viper2::$smarty -> register_outputfilter(array($this,'modifierPageTitle'));
      Viper2::$smarty -> register_function('NormalizeName',array($this,'functionNormalizeName'), false, array('name'));
      Viper2::$smarty -> register_block('dynamic', array($this, 'blockDynamic'), false);
      Viper2::$smarty -> register_function('Money',array($this,'functionMoney'), false, array('frm','number'));
      Viper2::$smarty -> register_modifier('Money',array($this,'modifierMoney'));
      Viper2::$smarty -> register_modifier('round_0_01',array($this,'modifierRound_0_01'));
      Viper2::$smarty -> register_modifier('round_1',array($this,'modifierRound_1'));
      Viper2::$smarty -> register_block('highlightKeywords',array($this,'highlightKeywords'));
      Viper2::$smarty -> register_modifier('Tydny',array($this,'modifierTydny'));
      Viper2::$smarty -> register_modifier('_',array($this,'modifierGettext'));
      Viper2::$smarty -> register_modifier('format_date',array($this,'modifierFormatDate'));
      Viper2::$smarty -> register_modifier('mb_string_format',array($this,'modifierMbStringFormat'));
      Viper2::$smarty -> register_function('MicroTime',array($this,'functionMicroTime'), false, array('name'));
    }

  }
  
  function actionGlobal() {
    Viper2::$smarty -> assign("basePage", Viper2::$router->basePage());
    Viper2::$smarty -> assign("pageUri", Viper2::$router->uri());
    Viper2::$smarty -> assign("language", _LANGUAGE);
    Viper2::$smarty -> assign("SID", session_id());

    Viper2::$smarty -> assign("pageTitle", $this->pageTitle);
    Viper2::$smarty -> assign("pageDescription", $this->pageDescription);
    Viper2::$smarty -> assign("pageId", $this->pageId);
    Viper2::$smarty -> assign("attPath", $this->attPath);
  }

  /**
   * @layout: "404.tpl"
   */
  public function action404() {
    Viper2::setLayout("404.tpl");
    header("HTTP/1.0 404 Not Found");
  }

  function modifierFormatDate($DateTime) {
    if (defined('_LANGUAGE')) {
      $Lang = _LANGUAGE;
    } else {
      $Lang = 'cs';
    }
    switch ($Lang) {
      case 'en':
        if (preg_match('/(\d{4})-(\d{2})-(\d{2})( \d+:\d+:\d+)?/', $DateTime, $res)) {
          if (isset($res[4])) {
            $Time = ' at'.$res[4];
          } else {
            $Time = '';
          }
          return $res[2].'/'.$res[3].'/'.$res[1].$Time;
        }
        break;
      case 'cs':
      default:
        if (preg_match('/(\d{4})-(\d{2})-(\d{2})( \d+:\d+:\d+)?/', $DateTime, $res)) {
          if (isset($res[4])) {
            $Time = ' v'.$res[4];
          } else {
            $Time = '';
          }
          return $res[3].'.'.$res[2].'.'.$res[1].$Time;
        }
        break;
    }
    if (preg_match('/(\d{4})-(\d{2})-(\d{2})( \d+:\d+:\d+)?/', $DateTime, $res)) {
      if (isset($res[4])) {
        $Time = ' v'.$res[4];
      } else {
        $Time = '';
      }
      return $res[3].'.'.$res[2].'.'.$res[1].$Time;
    }
    return $DateTime;
  }
  
  function modifierPageTitle($tpl_output, &$smarty) {
    // pokusime se filtr aplikovat jen na finalni template kvuli optimalizaci
    if (Viper2::$renderFinished) {
      $vars=$smarty->get_template_vars();
      $tpl_output =
      preg_replace('/##TITLE##/',
                      htmlspecialchars($vars["pageTitle"]), $tpl_output);
      $tpl_output =
      preg_replace('/##DESCRIPTION##/',
                      htmlspecialchars($vars["pageDescription"]), $tpl_output);
      $tpl_output =
         preg_replace('/##VIPER_CSS##/',
                      Viper2::processCSS(), $tpl_output);
      $tpl_output =
         preg_replace('/##VIPER_JS##/',
                      Viper2::processJS(), $tpl_output);
      // if ($vars['USE_CATS_KEYWORDS']) {
      //   if (preg_match('/<!-- keywords:([^\-]+)-->/',$tpl_output,$res)) {
      //     foreach($res as $Keywords) {
      //       viper::addKeywords($Keywords);
      //     }
      //   }
      // }
      // $keywords = viper::getKeywordString();
      // if (empty($keywords)) {
      //   $keywords = DEFAULT_KEYWORDS;
      // }
      // $tpl_output =
      //    preg_replace('/##VIPER_KEYWORDS##/',
      //                 $keywords, $tpl_output);
      $tpl_output =
         preg_replace('/##VIPER_KEYWORDS##/',
                      "", $tpl_output);
    }
    return($tpl_output);
  }

  function functionNormalizeName($params, &$smarty) {
    return(Viper2::$router->normalizeName($params["name"]));
  }

  function blockDynamic($param, $content, &$smarty) {
    return($content);
  }

  function functionMoney($params, &$smarty) {
    if ($params["number"]=="") {
      $params["number"]=0;
    }
    return(preg_replace("/\,0+/",",-".$params["spacer"],money_format($params["frm"],round(10*$params["number"])/10)));
  }

  function modifierMoney($number, $frm="%!.2n", $spacer=" Kč") {
    if ($number=="") {
      $number=0;
    }
    return(preg_replace("/\,0+/",",-",money_format($frm,round(10*$number)/10)).$spacer);
  }
  
  function modifierRound_0_01($number) {
    if ($number=="") {
      $number=0;
    }
    return(round(100*$number)/100);
  }
  
  function modifierRound_1($number) {
    if ($number=="") {
      $number=0;
    }
    return(round($number));
  }
  
  function highlightKeywords($params, $content, &$s, &$repeat) {
    if (empty($content)) {
      return;
    }
    if (empty($params['keywords'])) {
      return($content);
    }
    // mame nejaka klicova slova k highlightu
    $DefaultColors = array (
                      array('BgColor'=>'#ffff66', 'Color'=>'#000000'),
                      array('BgColor'=>'#a0ffff', 'Color'=>'#000000'),
                      array('BgColor'=>'#99ff99', 'Color'=>'#000000'),
                      array('BgColor'=>'#ff9999', 'Color'=>'#000000'),
                      array('BgColor'=>'#ff66ff', 'Color'=>'#000000'),
                      array('BgColor'=>'#880000', 'Color'=>'#ffffff'),
                      array('BgColor'=>'#00aa00', 'Color'=>'#ffffff'),
                      array('BgColor'=>'#886800', 'Color'=>'#ffffff'),
                      array('BgColor'=>'#004699', 'Color'=>'#ffffff'),
                      array('BgColor'=>'#990099', 'Color'=>'#ffffff')
                    );

    $KeyWordsParts = explode(';',$params['keywords']);
    $KeyWordsWithColor = array();
    foreach($KeyWordsParts as $KeyWordDef) {
      if (preg_match('/^(.*),(#[0-9a-fA-F]{6}|[a-zA-Z])$/u', $KeyWordDef, $res)) {
        $KeyWordRegex = CzechRE(preg_quote($res[1]));
        $BgColor = $res[2];
        if (array_key_exists($BgColor, $DefaultColors)) {
          $Color=$DefaultColors[$BgColor];
          unset($DefaultColors[$BgColor]);
        } else {
          // nutno dopocitat $Color
          $Color='#ffffff';
        }
        $KeyWordsWithColor[$KeyWordRegex] = array('BgColor'=>$BgColor, 'Color'=>$Color);
      } else {
        $KeyWordsWithColor[CzechRE(preg_quote($KeyWordDef))] = '';
      }
    }
    // ted pro klicova slova, kde neurcil barvu, zvolime barvu z nepouzitych
    foreach ($KeyWordsWithColor as $KeyWord=>$ColorDefinition) {
      if (empty($ColorDefinition)) {
        if (empty($DefaultColors)) {
          // nejsou nadefinovane barvy, zvolime nejaky zaklad
          $Color = array('BgColor'=>'#ffff66', 'Color'=>'#000000');
        } else {
          $Color = current($DefaultColors);
          // a soupneme poiner o 1 barvu dale
          if (!next($DefaultColors)) {
            // jsme na konci pole
            reset($DefaultColors);
          }
        }
        $KeyWordsWithColor[$KeyWord] = $Color;
      }
    }
    // tak a konecne nahrazeni
    
    foreach ($KeyWordsWithColor as $KeyWord=>$Color) {
      $content = preg_replace('/(^|>)([^<]*)('.$KeyWord.')([^<]*)(<|$)/iu','$1$2<span style="color:'.$Color['Color'].'; background-color:'.$Color['BgColor'].'">$3</span>$4$5',$content);
    }
    return $content;
  }
  
  function modifierTydny($Days=NULL) {
    if (!isset($Days)) {
      $Days=0;
    }
    if ($Days<7) {
      $res.=$Days." ";
      if($Days==1){
        $res.=_("dne");
      } else {
        $res.=_("dnů");
      }
    } else {
      $Weeks=floor($Days/7);
      $res="";
      if($Weeks){
        if($Weeks==1){
          $res=_("týdne");
        } else {
          $res=($Weeks*1)._(" týdnů ");
        }
      }
    }
    return($res);
  }

  function modifierGettext($inputText, $number=null) {
    if (is_null($number)) {
      return _($inputText);
    } else {
      return sprintf(ngettext($inputText, $inputText.' plural', $number), $number);
    }
  }
  
  function modifierMbStringFormat($string,$format) {
    return(mb_vsprintf($format,array($string)));
  }

  function functionMicroTime($params, &$smarty) {
    if(empty($params['var'])){
      return(microtime_float());
    } else {
      $smarty->assign($params['var'],microtime_float());
    }
  }
  
  /**
   * @layout: "maintenance.tpl"
   */
  public function actionMaintenance() {
    Viper2::setLayout("maintenance.tpl");
    header('HTTP/1.1 503 Service Temporarily Unavailable');
    header('Status: 503 Service Temporarily Unavailable');
    header('Retry-After: 3600');
  }
  
  
}
?>